@@ -221,6 +221,33 @@ sub testCompile($$$) {
 
 
 
+sub testCompileLink($$$) {
+    my ($flags, $cSourceCodeR, $successR) = @_;
+#-----------------------------------------------------------------------------
+#  Do a test compile of the program in @{$cSourceCodeR}.
+#  
+#  Return $$successR == $TRUE iff the compile succeeds (exit code 0).
+#-----------------------------------------------------------------------------
+    my ($cFile, $cFileName) = tempFile('.c');
+
+    print $cFile @{$cSourceCodeR};
+    
+    my ($oFile, $oFileName) = tempFile('');
+    
+    my $compileCommand = "$testCc -o $oFileName $flags $cFileName";
+    print ("Doing test compile/link: $compileCommand\n");
+    my $rc = system($compileCommand);
+    
+    unlink($oFileName);
+    close($oFile);
+    unlink($cFileName);
+    close($cFile);
+
+    $$successR = ($rc == 0);
+}
+
+
+
 sub displayIntroduction() {
     print("This is the Netpbm configurator.  It is an interactive dialog " .
           "that\n");
@@ -584,10 +611,12 @@ sub libSuffix($) {
 #-----------------------------------------------------------------------------
     my $suffix;
 
-    if ($platform eq "windows") {
-        $suffix = ".a";
+    if ($platform eq 'windows') {
+        $suffix = '.a';
+    } elsif ($platform eq 'darwin') {
+        $suffix = '.dylib';
     } else {
-        $suffix = ".so";
+        $suffix = '.so';
     }
 }
 
@@ -999,8 +1028,8 @@ sub getX11Library($@) {
 
         if (-d('/usr/link/X11')) {
             $default = '/usr/link/X11/libX11' . libSuffix($platform);
-        } elsif (-d('/usr/lib/X11')) {
-            $default = '/usr/lib/libX11' . libSuffix($platform);
+        } elsif (-d('/usr/X11R6')) {
+            $default = '/usr/X11R6/lib/libX11' . libSuffix($platform);
         } else {
             $default = "libX11" . libSuffix($platform);
         }
@@ -1464,6 +1493,34 @@ sub testPngHdr($$$) {
 
 
 
+sub testLinkPnglib($$) {
+    my ($generalCflags, $pngCflags) = @_;
+
+    my @cSourceCode = (
+                       "#include <png.h>\n",
+                       "int main() {\n",
+                       "png_create_write_struct(0, NULL, NULL, NULL);\n",
+                       "}\n",
+                       );
+
+    testCompile("$generalCflags $pngCflags", \@cSourceCode, \my $success);
+    if (!$success) {
+        # Since it won't compile, we can't test the link
+    } else {
+        my $pngLdflags = qx{libpng-config --ldflags};
+        chomp($pngLdflags);
+        
+        testCompileLink("$generalCflags $pngCflags $pngLdflags",
+                        \@cSourceCode, \my $success);
+        
+        if (!$success) {
+            printBadPngConfigLdflagsWarning($pngLdflags);
+        }
+    }
+}
+
+
+
 sub testLibpngConfig($) {
     my ($needLocal) = @_;
 #-----------------------------------------------------------------------------
@@ -1478,15 +1535,9 @@ sub testLibpngConfig($) {
     testCompilePngH("$generalCflags $pngCflags", \my $success);
 
     if (!$success) {
-        print("\n");
-        print("Unable to compile a test PNG program using the compiler " .
-              "flags that libpng-config says to use: '$pngCflags'.\n");
-        print("This indicates that you have Libpng installed incorrectly.\n");
-        print("libpng-config gets installed as part of the Libpng package.\n");
+        printBadPngConfigCflagsWarning($pngCflags);
     } else {
-        
-
-
+        testLinkPnglib($generalCflags, $pngCflags);
     }
 }
 
@@ -1976,7 +2027,7 @@ if ($platform eq "GNU") {
     push(@Makefile_config, 'CFLAGS_SHLIB = -fno-common', "\n");
     push(@Makefile_config, "LDSHLIB = ",
          "-dynamiclib ",
-         '-install_name $(NETPBM_RUNTIME_PATH)/libnetpbm.$(MAJ).dylib', 
+         '-install_name $(NETPBMLIB_RUNTIME_PATH)/libnetpbm.$(MAJ).dylib', 
          "\n");
 #    push(@Makefile_config, "INSTALL = install\n");
 } else {
