@@ -18,15 +18,13 @@
 #include <assert.h>
 #include <stdlib.h>
 
+#include "pm_config.h"
 #include "pm_c_util.h"
 #include "mallocvar.h"
 #include "ppm.h"
 #include "ppmdfont.h"
 #include "ppmdraw.h"
 
-typedef int qsort_compare(const void *, const void *);
-    /* A compare function to pass to stdlib.h's qsort() */
-
 
 #define DDA_SCALE 8192
 
@@ -779,7 +777,9 @@ ppmd_fill_drawproc(pixel**      const pixels,
 
 
 
-static qsort_compare yx_compare;
+#ifndef LITERAL_FN_DEF_MATCH
+static qsort_comparison_fn yx_compare;
+#endif
 
 static int
 yx_compare(const void * const c1Arg,
@@ -941,12 +941,7 @@ static int extleft, exttop, extright, extbottom;  /* To accumulate extents */
 /*  ISIN  --  Return sine of an angle in integral degrees.  The
           value returned is 65536 times the sine.  */
 
-#if __STDC__
 static long isin(int deg)
-#else
-    static long isin(deg)
-    int deg;
-#endif
 {
     /* Domain reduce to 0 to 360 degrees. */
 
@@ -971,12 +966,7 @@ static long isin(int deg)
 /*  ICOS  --  Return cosine of an angle in integral degrees.  The
           value returned is 65536 times the cosine.  */
 
-#if __STDC__
 static long icos(int deg)
-#else
-    static long icos(deg)
-    int deg;
-#endif
 {
     return isin(deg + 90);
 }  
