@@ -65,7 +65,7 @@ extern "C" {
 #ifdef __GNUC__
 #define PM_GNU_PRINTF_ATTR(a,b) __attribute__ ((format (printf, a, b)))
 #else
-#define PM_GNU_PRINTF_ATTR
+#define PM_GNU_PRINTF_ATTR(a,b)
 #endif
 
 
@@ -348,6 +348,9 @@ pm_check(FILE *               const file,
 char *
 pm_arg0toprogname(const char arg0[]);
 
+unsigned int
+pm_randseed(void);
+
 #ifdef __cplusplus
 }
 #endif
